import signal
import logging
import argparse

from . import command

def main():
    prog = 'guardctrl'
    description = """Guardian process control cli.

Add '-h' after subcommand for subcommand help.
"""
    parser = argparse.ArgumentParser(prog=prog,
                                     description=description,
                                     formatter_class=argparse.RawDescriptionHelpFormatter,
                                     )
    parser.add_argument('--debug-query', action='store_true', help=argparse.SUPPRESS)

    subparser = parser.add_subparsers(
        metavar='COMMAND',
        dest='cmd',
        )

    for cmd, mod in command.command_modules.iteritems():
        sp = subparser.add_parser(cmd,
                                  help=mod.SUMMARY,
                                  formatter_class=argparse.RawDescriptionHelpFormatter,
                                  )
        mod.gen_argparser(sp)

    args = parser.parse_args()
    cmd = args.cmd
    del args.cmd

    logging.debug('cmd: %s' % (cmd))
    logging.debug('args: %s' % (args))

    signal.signal(signal.SIGINT, signal.SIG_DFL)
    signal.signal(signal.SIGPIPE, signal.SIG_DFL)

    mod = command.command_modules[cmd]
    args = mod.validate_args(args)
    for out in mod.execute_args(args):
        print(out)

if __name__ == '__main__':
    main()
