import SocketServer
import BaseHTTPServer
import SimpleHTTPServer
import urlparse
import argparse
import logging
import traceback

from . import const
from . import util
from . import command

##################################################

def parse_query(query):
    ad = dict()
    for k,v in query.iteritems():
        if k == 'nodes':
            ad[k] = v
        else:
            ad[k] = v[0]
    return argparse.Namespace(**ad)

def format_args(args):
    for k,v in vars(args).iteritems():
        yield 'QUERY: {}: {}'.format(k,v)

##################################################

class GuardCtrlHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    error_content_type = 'text/plain'

    def process_request(self, head_only=False):
        logging.debug("request received: {}".format(self.path))

        try:
            o = urlparse.urlparse(self.path)
            query = urlparse.parse_qs(o.query, keep_blank_values=True)
        except:
            self.send_error(400, "Malformed query")
            raise

        cmd = o.path.strip('/')

        logging.debug("parsed query: {} {}".format(cmd, query))

        try:
            mod = command.command_modules[cmd]
        except KeyError:
            self.send_error(404, "Unknown command: {}".format(cmd))
            return

        try:
            args = parse_query(query)
        except AttributeError as e:
            self.send_error(400, "Query error: {}".format(e))
            raise
        try:
            args = mod.validate_args(args)
            func = mod.execute_args
        except util.GuardCtrlError as e:
            self.send_error(400, "Query error: {}".format(e))
            raise
        except:
            self.send_error(500, "Internal server error during command parsing.  Please contact admin.")
            raise

        logging.debug("validated args: {} {}".format(cmd, args))

        self.send_response(200)
        self.send_header("Content-type", "text/plain")
        self.send_header("Cache-Control", "no-cache, no-store, must-revalidate")
        self.send_header(const.SERVER_ERROR_HEADER, const.SERVER_ERROR_STRING)
        self.end_headers()
        if head_only:
            return

        if 'debug_query' in args and args.debug_query:
            func = format_args

        ##########
        # handle requests
        try:
            for line in func(args):
                self.wfile.write(line+'\n')
        except:
            #self.wfile.write(traceback.format_exc())
            self.wfile.write(const.SERVER_ERROR_STRING+'\n')
            raise
        ##########

        logging.debug("request processed.")

    def do_HEAD(self):
        return self.process_request(head_only=True)

    def do_GET(self):
        return self.process_request()


class ForkingServer(SocketServer.ForkingMixIn, BaseHTTPServer.HTTPServer):
    pass

##################################################

if __name__ == '__main__':

    import os
    import signal

    address = ('', const.GUARDCTRL_PORT)

    os.setpgrp()

    logging.info("starting guardctrl server: %s:%s..." % address)

    server = ForkingServer(address, GuardCtrlHandler)

    def terminate(signum, *args):
        logging.info("caught signal %s; terminating" % signum)
        raise SystemExit

    # signal.signal(signal.SIGINT, terminate)
    signal.signal(signal.SIGTERM, terminate)
    signal.signal(signal.SIGPIPE, signal.SIG_DFL)

    try:
        server.serve_forever()
    except KeyboardInterrupt:
        pass
    except SystemExit:
        pass
    finally:
        logging.info("shutting down...")
        server.shutdown()
        server.server_close()
        logging.info("done.")
        os.killpg(0, signal.SIGTERM)
