import os
import gzip
import fnmatch

from . import const

##########

class GuardCtrlError(Exception):
    pass

##########

def full_node_list():
    """return list of all active nodes"""
    return os.listdir(const.GUARD_NODE_DIR)

def nodes_expand(glob_list=None):
    """return list of nodes matching patterns

    Match a list of globbed node names with the full node list,
    returning a list of matching nodes.  If no glob list is provided
    all nodes will be returned.

    An exception is raised if a pattern matches no known nodes.

    """
    node_list = full_node_list()
    if not glob_list:
        return sorted(node_list)
    expanded = set()
    for glob in glob_list:
        matches = fnmatch.filter(node_list, glob)
        if not matches:
            raise GuardCtrlError("Pattern {} matches no known nodes".format(glob))
        expanded |= set(matches)
    return sorted(expanded)

##########

def in_interval(t, after=None, before=None):
    """determine if time within interval"""
    ab = True
    bb = True
    if after and t < after:
        ab = False
    if before and t > before:
        bb = False
    return ab and bb

##########

GZIP_MAGIC_NUMBER = '\x1f\x8b\x08'

def opportunistically_decompress(filename, mode='r'):
    """trick to opportunistically decompress gzip files

    Opens filename and looks for the gzip "magic number" in the first
    couple of bytes.  If it is not found the open file object is
    returned.  If it is found, the file object is un-gziped and then
    returned.

    """
    fd = open(filename, mode)
    # gzip magic number
    data = fd.read(len(GZIP_MAGIC_NUMBER))
    fd.seek(0)
    if data != GZIP_MAGIC_NUMBER:
        return fd
    else:
        return gzip.GzipFile(filename, mode, fileobj=fd)
